/**
 * Blended linearized wave-equation (Born)
 * modeling operator of acoustic particle
 * velocity data.
 * @author: Joseph Jennings
 * @version: 2017.05.14
 */
#ifndef BBORNPV_H_
#define BBORNPV_H_

#include <vector>
#include "my_operator.h"
#include "axis.h"
#include "hypercube_float.h"
#include "sregf.h"

class bbornpv : public my_operator {
  public:
    bbornpv(){};
    bbornpv(std::vector<axis> paxes, std::vector<axis> daxes, axis saxis, int srcz, int recz,
        hypercube_float *bvel, hypercube_float *src, float rho, float dtu, float fmax,
        int minblnd, int maxlbnd, float mindly, float maxdly, int seed, int space);
    bbornpv(std::vector<axis> paxes, std::vector<axis> daxes, int srcz, int recz,
        hypercube_float *bvel, hypercube_float *src, float rho, float dtu, float fmax,
        int minblnd, int maxblnd, std::vector<float> stimes, std::vector<float> xss);
    int get_nbshots(){ return (int)_nb.size(); };
    void setbwfldout(std::string wfname, int wfldj);
    void setawfldout(std::string wfname, int wfldj);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    ~bbornpv(){};
  private:
    std::vector<axis> _paxes, _daxes, _raxes;
    std::vector<float> _stimei, _xsi;
    std::vector<std::vector<float>> _stimes, _xss;
    std::vector<int> _nb;
    axis _saxis;
    int _recz, _srcz, _nx, _nz, _nsx, _nt, _nrx, _wfldj;
    hypercube_float *_bvel, *_src, *_imgs;
    float _osx = 0.0, _dsx = 1.0;
    float _rho, _dtu, _fmax;
    std::string _bwfname, _awfname;
    sregf _wfout, _awfout;
};


#endif /* BORNPV_H_ */
