/*
 * blendnc2d.cpp
 *
 *  Created on: Apr 06, 2017
 *      Author: joe
 */

#include "blendnc2d.h"
#include <random>

blendnc2d::blendnc2d(int ns, int minblnd, int maxblnd) {
  /* Calculate nb */
  _ns = ns;
  _nb = calc_nb(minblnd,maxblnd);
}

blendnc2d::blendnc2d(std::vector<axis> maxes, std::vector<float> stimes, int minblnd, int maxblnd) {
  _nt = maxes[0].n; _nrx = maxes[1].n; _ns = maxes[2].n;
  std::vector<axis> osaxes;
  osaxes.push_back(maxes[0]);
  osaxes.push_back(maxes[1]);
  osaxes.push_back(axis(1,0.0,1.0));
  _eshts = new hypercube_float(maxes,true);
  _osht = new hypercube_float(osaxes,true);
  _esht = new hypercube_float(osaxes,true);
  _sp = shotpad(maxes[0]);
  if((int)stimes.size() != _ns) {
    seperr("blendnc2d: Please enter a shot time for each shot %d %d.\n",(int)stimes.size(),maxes[2].n);
  }
  for(int is = 0; is < _ns; ++is) {
    if(stimes[is] > _nt*maxes[0].d) {
      seperr("blendnc2d: Shot times cannot be larger than individual shot time\n");
    }
  }
  _stimes = stimes;
  /* Calculate nb */
  _nb = calc_nb(minblnd,maxblnd);
  _run_nshots = false;
}

blendnc2d::blendnc2d(std::vector<axis> maxes, float maxdev, float mindev, int minblnd, int maxblnd) {
  _nt = maxes[0].n; _nrx = maxes[1].n; _ns = maxes[2].n;
  if(maxdev > _nt*maxes[0].d || mindev > _nt*maxes[0].d) {
    seperr("blendnc2d: Min or max delay cannot be larger than individual shot time %f %f\n",mindev,maxdev);
  }
  std::random_device rd; std::mt19937 rng(rd());
  std::uniform_real_distribution<float> unif(maxdev,mindev);
  /* First time is always zero */
  _stimes.push_back(0.0);
  for(int is = 1; is < _ns; ++is) {
    auto val = unif(rng);
    _stimes.push_back(val);
  }
  std::vector<axis> osaxes;
  osaxes.push_back(maxes[0]);
  osaxes.push_back(maxes[1]);
  osaxes.push_back(axis(1,0.0,1.0));
  _eshts = new hypercube_float(maxes,true);
  _osht = new hypercube_float(osaxes,true);
  _esht = new hypercube_float(osaxes,true);
  _sp = shotpad(maxes[0]);
  /* Calculate nb */
  _nb = calc_nb(minblnd,maxblnd);
  _run_nshots = false;
}

blendnc2d::blendnc2d(std::vector<axis> maxes, float maxdev, float mindev, int seed, int minblnd, int maxblnd) {
  _nt = maxes[0].n; _nrx = maxes[1].n; _ns = maxes[2].n;
  if(maxdev > _nt*maxes[0].d || mindev > _nt*maxes[0].d) {
    seperr("blendnc2d: Min or max delay cannot be larger than individual shot time %f %f\n",mindev,maxdev);
  }
  std::mt19937 rng(seed);
  std::uniform_real_distribution<float> unif(maxdev,mindev);
  /* First time is always zero */
  _stimes.push_back(0.0);
  for(int is = 1; is < _ns; ++is) {
    auto val = unif(rng);
    _stimes.push_back(val);
  }
  std::vector<axis> osaxes;
  osaxes.push_back(maxes[0]);
  osaxes.push_back(maxes[1]);
  osaxes.push_back(axis(1,0.0,1.0));
  _eshts = new hypercube_float(maxes,true);
  _osht = new hypercube_float(osaxes,true);
  _esht = new hypercube_float(osaxes,true);
  _sp = shotpad(maxes[0]);
  /* Calculate nb */
  _nb = calc_nb(minblnd,maxblnd);
  _run_nshots = false;
}

//TODO: Write a unit test for this function
// Only works for 2 and 3 shots
std::vector<int> blendnc2d::calc_nb(int minblnd, int maxblnd) {
  std::vector<int> nb;
  std::random_device rd; std::mt19937 rng(rd());
  std::uniform_int_distribution<int> unii(minblnd,maxblnd);
  int nss = _ns; bool mm = false; int sum = 0;
  if(maxblnd == minblnd && _ns%maxblnd != 0) {
    for(int is = 0; is < _ns/maxblnd; ++is) {
      nb.push_back(maxblnd);
      sum += maxblnd;
    }
    nb.push_back(1);
    sum += 1;
  } else {
    while(nss > maxblnd) {
      if(nss - maxblnd == 1) {
        mm = true;
        break;
      }
      auto val = unii(rng);
      nss -= val; sum += val;
      nb.push_back(val);
    }
    if(mm) {
      nb.push_back(minblnd);
      sum += 2*minblnd;
      nb.push_back(minblnd);
    } else if(_ns - sum == minblnd) {
      nb.push_back(minblnd);
      sum += minblnd;
    } else {
      nb.push_back(maxblnd);
      sum += maxblnd;
    }
  }
  if(sum != _ns) seperr("Error: Blended ranges are incorrect sum=%d.\n",sum);
  return nb;
}

int blendnc2d::get_nshots() {
  _run_nshots = true;
  return (int)_nb.size();
}

//TODO: Write out the encoded shots
bool blendnc2d::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  /* Check if they have run get_nshots */
  if(!_run_nshots) seperr("blend: Please run_nshots to allocate the new number of shots "
      "for your output blended data.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Pad each shot */
  for(int is = 0; is < _ns; ++is) {
    memcpy(_osht->vals,&m->vals[_nt*_nrx*is],sizeof(float)*_nt*_nrx);
    _sp.set_time(_stimes[is]);
    _sp.forward(false,_osht,_esht);
    memcpy(&_eshts->vals[_nt*_nrx*is],_esht->vals,sizeof(float)*_nt*_nrx);
  }

  /* Add the padded shots */
  int k = 0;
  for(int is = 0; is < (int)_nb.size(); ++is) {
    _osht->zero();
    for(int ib = 0; ib < _nb[is]; ++ib) {
      memcpy(_esht->vals,&_eshts->vals[_nt*_nrx*(k)],sizeof(float)*_nt*_nrx);
      _osht->scale_add(1.0,_esht,1.0);
      ++k;
    }
    memcpy(&d->vals[_nt*_nrx*is],_osht->vals,sizeof(float)*_nt*_nrx);
  }

  return true;
}

void blendnc2d::get_encshots(hypercube_float *eshts) {
  if(_eshts != nullptr) {
    if(!_eshts->check_match(eshts)) {
      seperr("The size of the input encoded shots does not match the computed "
          " size of computed encoded shots.\n");
    }
    eshts->set(_eshts->vals);
  }
}

bool blendnc2d::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();
  return true;
}
