/**
 * Blends shot records in a non-continous
 * manner (cuts the ends)
 * @author: Joseph Jennings
 * @version: 2017.04.05
 */
#ifndef BLENDNC2D_H_
#define BLENDNC2D_H_

#include "my_operator.h"
#include "hypercube_float.h"
#include "shotpad.h"

class blendnc2d : public my_operator {
  public:
    blendnc2d(){};
    blendnc2d(int ns, int minblnd, int maxblnd);
    blendnc2d(std::vector<axis> maxes, std::vector<float> stimes, int minblnd, int maxblnd);
    blendnc2d(std::vector<axis> maxes, float maxdev, float mindev, int minblnd, int maxblnd);
    blendnc2d(std::vector<axis> maxes, float maxdev, float mindev, int seed, int minblnd, int maxblnd);
    int get_nshots();
    virtual bool forward(bool add, my_vector *mod, my_vector *dat);
    virtual bool adjoint(bool add, my_vector *mod, my_vector *dat);
    void get_encshots(hypercube_float *eshts);
    std::vector<int> get_nb(){ return _nb;};
    std::vector<int> calc_nb(int minblnd, int maxblnd);
    ~blendnc2d(){
      delete _osht; delete _esht; delete _eshts;
    };

  private:
    int _ns, _nt, _nrx;
    hypercube_float *_osht, *_esht, *_eshts;
    shotpad _sp;
    std::vector<float> _stimes;
    std::vector<int> _nb;
    bool _run_nshots;
};

#endif /* BLENDNC2D_H_ */
