/*
 * cghelp.cpp
 *
 *  Created on: May 4, 2017
 *      Author: joe
 */


#include <math.h>
#include <vector>
#include "axis.h"
#include "sregf.h"
#include "super_vector.h"
#include "cghelp.h"
#include "simpleplot.h"

cghelp::cghelp(std::string modtag, std::string grdtag, std::string oftag, std::string ddtag) {
  _fouts["mod"] = modtag; _fouts["grd"] = grdtag; _fouts["ofn"] = oftag; _fouts["dbl"] = ddtag;

}


void cghelp::set_resout(std::string tag) {
  _fouts["rsd"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_mrsout(std::string tag) {
  _fouts["rsm"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_grdout(std::string tag) {
  _fouts["grd"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_grd1out(std::string tag) {
  _fouts["grd1"] = tag;
  if(_fouts["grd"].empty()) {
    _fouts["grd"] = "grddummy";
  }
}

/**
 *
 * @param tag
 */
void cghelp::set_grd2out(std::string tag) {
  _fouts["grd2"] = tag;
  if(_fouts["grd"].empty()) {
    _fouts["grd"] = "grddummy";
  }
}

/**
 *
 * @param tag
 */
void cghelp::set_modout(std::string tag) {
  _fouts["mod"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_mod1out(std::string tag) {
  _fouts["mod1"] = tag;
  if(_fouts["mod"].empty()) {
    _fouts["mod"] = "moddummy";
  }
}

/**
 *
 * @param tag
 */
void cghelp::set_mod2out(std::string tag) {
  _fouts["mod2"] = tag;
  if(_fouts["mod"].empty()) {
    _fouts["mod"] = "moddummy";
  }
}

/**
 *
 * @param tag
 */
void cghelp::set_rdnout(std::string tag) {
  _fouts["rdn"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_rmnout(std::string tag) {
  _fouts["rmn"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_rtnout(std::string tag) {
  _fouts["rtn"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_gnout(std::string tag) {
  _fouts["gdn"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_ofout(std::string tag) {
  _fouts["ofn"] = tag;
}

/**
 *
 * @param tag
 */
void cghelp::set_ddout(std::string tag) {
  _fouts["dbl"] = tag;
}

void cghelp::output(my_vector *res, my_vector *ddat, my_vector *grd, my_vector *mod, double obj) {
  _counter += 1;
  axis a1d = axis(_counter, 0, 1.0, "iteration");
  if(_fouts.size() != 0) {
    /* First check for residuals (data and model)*/
    if(!_fouts["rsd"].empty() || !_fouts["rsm"].empty()) {
      /*  Check if regularized inversion */
      if(!res->name.compare("super_vector")) {
        if(!_fouts["rsd"].empty()) {
          /* Write data space residual */
          super_vector *rv1   = (super_vector*)res;
          hypercube_float *rd = (hypercube_float*)rv1->vecs[0];
          sregf rdout = sregf(rd->get_axes());
          rdout.write_description(_fouts["rsd"]);
          rdout.write_all(_fouts["rsd"],rd);
          rdout.write_axis(_fouts["rsd"],rd->get_ndim()+1,a1d);
        }
        if(!_fouts["rsm"].empty()) {
          /* Write model space residual */
          super_vector *rv2   = (super_vector*)res;
          hypercube_float *rm = (hypercube_float*)rv2->vecs[1];
          sregf rmout = sregf(rm->get_axes());
          rmout.write_description(_fouts["rsm"]);
          rmout.write_all(_fouts["rsm"],rm);
          rmout.write_axis(_fouts["rsm"],rm->get_ndim()+1,a1d);
        }
      } else {
        /* If normal inversion, write data space residual */
        if(!_fouts["rsm"].empty())
          seperr("Cannot write model residual without regularization term.\n");
        hypercube_float *r = (hypercube_float*)res;
        sregf rout = sregf(r->get_axes());
        rout.write_description(_fouts["rsd"]);
        rout.write_all(_fouts["rsd"],r);
        rout.write_axis(_fouts["rsd"],r->get_ndim()+1,a1d);
      }
    }
    /* Check and write gradient */
    if(!_fouts["grd"].empty()) {
      if(!grd->name.compare("super_vector")) {
        if(!_fouts["grd1"].empty()) {
          /* Write gradient for model 1 */
          super_vector *gv1   = (super_vector*)grd;
          hypercube_float *g1 = (hypercube_float*)gv1->vecs[0];
          sregf g1out = sregf(g1->get_axes());
          g1out.write_description(_fouts["grd1"]);
          g1out.write_all(_fouts["grd1"],g1);
          g1out.write_axis(_fouts["grd1"],g1->get_ndim()+1,a1d);
        }
        if(!_fouts["grd2"].empty()) {
          /* Write gradient for model 2 */
          super_vector *gv2   = (super_vector*)grd;
          hypercube_float *g2 = (hypercube_float*)gv2->vecs[1];
          sregf g2out = sregf(g2->get_axes());
          g2out.write_description(_fouts["grd2"]);
          g2out.write_all(_fouts["grd2"],g2);
          g2out.write_axis(_fouts["grd2"],g2->get_ndim()+1,a1d);
        }
      } else {
        hypercube_float *g = (hypercube_float*)grd;
        sregf gout = sregf(g->get_axes());
        gout.write_description(_fouts["grd"]);
        gout.write_all(_fouts["grd"],g);
        gout.write_axis(_fouts["grd"],g->get_ndim()+1,a1d);
      }
    }
    /* Check and write model update */
    if(!_fouts["mod"].empty()) {
      if(!mod->name.compare("super_vector")) {
        if(!_fouts["mod1"].empty()) {
          /* Write updated model 1 */
          super_vector *mv1   = (super_vector*)mod;
          hypercube_float *m1 = (hypercube_float*)mv1->vecs[0];
          sregf m1out = sregf(m1->get_axes());
          m1out.write_description(_fouts["mod1"]);
          m1out.write_all(_fouts["mod1"],m1);
          m1out.write_axis(_fouts["mod1"],m1->get_ndim()+1,a1d);
        }
        if(!_fouts["mod2"].empty()) {
          /* Write updated model 2 */
          super_vector *mv2   = (super_vector*)mod;
          hypercube_float *m2 = (hypercube_float*)mv2->vecs[1];
          sregf m2out = sregf(m2->get_axes());
          m2out.write_description(_fouts["mod2"]);
          m2out.write_all(_fouts["mod2"],m2);
          m2out.write_axis(_fouts["mod2"],m2->get_ndim()+1,a1d);
        }
      } else {
        hypercube_float *m = (hypercube_float*)mod;
        sregf mout = sregf(m->get_axes());
        mout.write_description(_fouts["mod"]);
        mout.write_all(_fouts["mod"],m);
        mout.write_axis(_fouts["mod"],m->get_ndim()+1,a1d);
      }
    }
    /* Check and write deblended data */
    if(!_fouts["dbl"].empty()) {
      hypercube_float *dd = (hypercube_float*)ddat;
      sregf ddout = sregf(dd->get_axes());
      ddout.write_description(_fouts["dbl"]);
      ddout.write_all(_fouts["dbl"],dd);
      ddout.write_axis(_fouts["dbl"],dd->get_ndim()+1,a1d);
    }
    /* Check and write residual norm */
    if(!_fouts["rdn"].empty() || !_fouts["rmn"].empty()) {
      if(!res->name.compare("super_vector")) {
        super_vector *rsup = (super_vector*)res;
        if(!_fouts["rdn"].empty()) {
          sregf rnout = sregf();
          std::vector<axis> raxis;
          raxis.push_back(a1d);
          rnout.set_axes(raxis);
          rnout.write_description(_fouts["rdn"]);
          _one.push_back(sqrt(rsup->vecs[0]->dot(rsup->vecs[0])));
          if(sizeof(float) != srite(_fouts["rdn"].c_str(),_one.data(), sizeof(float)))
            seperr("Could not write data residual norm\n");
        }
        if(!_fouts["rmn"].empty()) {
          sregf rnout = sregf();
          std::vector<axis> raxis;
          raxis.push_back(a1d);
          rnout.set_axes(raxis);
          rnout.write_description(_fouts["rmn"]);
          _one.push_back(sqrt(rsup->vecs[1]->dot(rsup->vecs[1])));
          if(sizeof(float) != srite(_fouts["rmn"].c_str(),_one.data(), sizeof(float)))
            seperr("Could not write model space residual norm\n");
        }
      }
      else {
        if(!_fouts["rmn"].empty())
          seperr("Cannot write model residual without regularization term.\n");
        sregf rnout = sregf();
        std::vector<axis> raxis;
        raxis.push_back(a1d);
        rnout.set_axes(raxis);
        rnout.write_description(_fouts["rdn"]);
        _one.push_back(sqrt(res->dot(res)));
        if(sizeof(float) != srite(_fouts["rdn"].c_str(),_one.data(), sizeof(float)))
          seperr("Could not write residual norm\n");
      }
    }
    if(!_fouts["rtn"].empty()) {
      sregf rnout = sregf();
      std::vector<axis> raxis;
      raxis.push_back(a1d);
      rnout.set_axes(raxis);
      rnout.write_description(_fouts["rtn"]);
      _one.push_back(sqrt(res->dot(res)));
      if(sizeof(float) != srite(_fouts["rtn"].c_str(),_one.data(), sizeof(float)))
        seperr("Could not write total residual norm\n");
    }
    /* Check and write gradient norm */
    if(!_fouts["gdn"].empty()) {
      sregf gnout = sregf();
      std::vector<axis> gaxis;
      gaxis.push_back(a1d);
      gnout.set_axes(gaxis);
      gnout.write_description(_fouts["gdn"]);
      _one.push_back(sqrt(grd->dot(grd)));
      if(sizeof(float) != srite(_fouts["gdn"].c_str(),_one.data(),sizeof(float)))
        seperr("Could not write gradient norm\n");
    }
    if(!_fouts["ofn"].empty()) {
      sregf ofnout = sregf();
      std::vector<axis> ofaxis;
      ofaxis.push_back(a1d);
      ofnout.set_axes(ofaxis);
      ofnout.write_description(_fouts["ofn"]);
      _one.push_back(obj);
      if(sizeof(float) != srite(_fouts["ofn"].c_str(),_one.data(),sizeof(float)))
        seperr("Could not write objective function value\n");
      _one.clear();
    }
  }
}
