/*
 * grad_taper.cpp
 *
 *  Created on: May 2, 2017
 *      Author: joe
 */

#include "grad_taper.h"
#include <math.h>

grad_taper::grad_taper(axis taxis, float z1, float z2) {
  /* Create taper function */
  _taxes.push_back(taxis);
  _tapfunc = new hypercube_float(_taxes,true);

  for(int iz = 0; iz < taxis.n; ++iz) {
    float z = iz*taxis.d + taxis.o;
    if(z < z1) {
      _tapfunc->vals[iz] = 0.0;
    } else if(z >= z1 && z < z2) {
      _tapfunc->vals[iz] = cos(M_PI/2.0 * (z2-z)/(z2-z1))*cos(M_PI/2.0 * (z2-z)/(z2-z1));
    } else if(z >= z2) {
      _tapfunc->vals[iz] = 1.0;
    }
  }
}

bool grad_taper::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!dat->check_match(mod)) seperr("grad_taper: model and data must be same size.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nz = m->get_axis(1).n;
  int nx = m->get_axis(2).n;

  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      d->vals[ix*nz + iz] += _tapfunc->vals[iz]*m->vals[ix*nz + iz];
    }
  }

  return true;
}

bool grad_taper::adjoint(bool add, my_vector *mod, my_vector *dat) {
  return true;
}
