/**
 * Independent Linearized wave-equation (Born)
 * modeling operator of acoustic particle
 * velocity data.
 * @author: Joseph Jennings
 * @version: 2017.05.14
 */
#ifndef INDBORNPV_H_
#define INDBORNPV_H_

#include <vector>
#include "my_operator.h"
#include "axis.h"
#include "hypercube_float.h"
#include "sregf.h"

class indbornpv : public my_operator {
  public:
    indbornpv(){};
    indbornpv(std::vector<axis> paxes, std::vector<axis> daxes, axis saxis, int srcz, int recz,
        hypercube_float *bvel, hypercube_float *src, float rho, float dtu, float fmax, std::vector<float> stimes);
    indbornpv(std::vector<axis> paxes, std::vector<axis> daxes, int srcz, int recz,
        hypercube_float *bvel, hypercube_float *src, float rho, float dtu, float fmax, 
        std::vector<float> stimes, std::vector<float> xss);
    void setbwfldout(std::string wfname, int wfldj);
    void setawfldout(std::string wfname, int wfldj);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    ~indbornpv(){};
  private:
    std::vector<axis> _paxes, _daxes, _raxes;
    std::vector<float> _stimes, _xss;
    axis _saxis;
    int _recz, _srcz, _nx, _nz, _nsx, _nt, _nrx, _wfldj;
    hypercube_float *_bvel, *_src, *_imgs;
    float _osx = 0.0, _dsx = 1.0;
    float _rho, _dtu, _fmax;
    std::string _bwfname, _awfname;
    sregf _wfout, _awfout;
};

#endif /* INDBORNPV_H_ */
