/*
 *
 *  Created on: May 1, 2017
 *      Author: joe
 */

#include <omp.h>
#include "mcpefdblnd.h"
#include "blendnc2d.h"
#include "pefsignoi3d.h"
#include "conj_grad.h"
#include "lv_conv3df.h"
#include "lv_conv3dm.h"
#include "identity.h"
#include "simpleplot.h"

mcpefdblnd::mcpefdblnd(int pefniter, int sniniter, float eps, std::vector<int> j, std::vector<int> nlag,
    float shtscale, int minblnd, int maxblnd, int nsx, std::vector<axis> daxes, int nthrds) {
  /* Inversion parameters */
  _pefniter = pefniter; _sniniter = sniniter; _eps = eps; _shtscale = shtscale;

  /* PEF Parameters */
  _j = j; _nlag = nlag;

  /* Build blending array */
  blendnc2d *gam = new blendnc2d(nsx,minblnd,maxblnd);
  _nb = gam->get_nb();

  if((int)_nb.size() != daxes[3].n) 
    seperr("mcpefdblnd: Calculated number of shots %d must be same as given %d.\n",(int)_nb.size(),daxes[3].n);
  _nt = daxes[0].n; _nrx = daxes[1].n; _nc = daxes[2].n;
  /* Create axis for a single shot */
  _saxes.push_back(daxes[0]);
  _ot = daxes[0].o; _dt = daxes[0].d;
  _saxes.push_back(daxes[1]);
  _orx = daxes[1].o; _drx = daxes[1].d;
  _saxes.push_back(daxes[2]);
  _oc = daxes[2].o; _dc = daxes[2].d;

  /* Create padded axis for a single shot */
  _ntp = _nt + nlag[0]/2;
  _paxes.push_back(axis(_ntp,_ot,_dt));
  _nrxp = _nrx + nlag[0]/2;
  _paxes.push_back(axis(_nrxp,_orx,_drx));
  _ncp = 3;
  _paxes.push_back(axis(_ncp,_oc,_dc));

  _nd.push_back(_ntp); _nd.push_back(_nrxp); _nd.push_back(_ncp);

  _pad = pad3d(0,0,0,0);

  /* Output data container */
  _odatcont = std::vector<std::vector<hypercube_float*>>((int)_nb.size(),std::vector<hypercube_float*>(2));

  /* Number of OpenMP threads */
  _nthrds = nthrds;
}

void mcpefdblnd::set_padding(int ntp, int nrxp, int ncp) {
  _ntp = ntp; _nrxp = nrxp; _ncp = ncp;
  _paxes[0] = axis(_ntp,_ot,_dt);
  _paxes[1] = axis(_nrxp,_orx,_drx);
  _paxes[2] = axis(_ncp,_oc,_dc);
}

void mcpefdblnd::set_smmvtags(std::vector<std::string> smmvtags) {
  _smmvtags = smmvtags;
}

void mcpefdblnd::set_nmmvtags(std::vector<std::string> nmmvtags) {
  _nmmvtags = nmmvtags;
}

void mcpefdblnd::find_pefs(int is, std::vector<hypercube_float*> iisht, std::vector<lv_conv3dm*> &pefops) {
  hypercube_float *psht   = new hypercube_float(_paxes,true);
  hypercube_float *idat   = new hypercube_float(_paxes,true);
  std::vector<hypercube_float*> flts;
  for(int ib = 0; ib < _nb[is]; ++ib) {
    fprintf(stderr,"ind shot %d\n",ib);
    /* Get and pad the shot */
    hypercube_float *ssht = iisht[ib];
    ssht->scale(_shtscale);
    _pad.forward(false,ssht,psht);

    /* Create convolution operator */
    lv_conv3df *lvf = new lv_conv3df(_nd,_j,_nlag,psht);

    /* Build model and data vectors */
    lvf->set_data(idat); idat->scale(-1.0);
    flts.push_back(new hypercube_float(lvf->get_filtaxes(),true));
    /* Initialize the filter */
    flts[ib]->zero(); lvf->set_zerolag(flts[ib]);
    hypercube_float *zro = new hypercube_float(lvf->get_filtaxes(),true);
    zro->zero();

    /* Create masked operator */
    lv_conv3dmask *k = new lv_conv3dmask();
    lvf->set_domain(flts[ib]); lvf->set_range(idat);
    k->set_domain(flts[ib]); k->set_range(flts[ib]);
    chain_op *dk = new chain_op(k,lvf);

    /* Create the identity operator */
    identity *iop = new identity();
    iop->set_domain(flts[ib]); iop->set_range(flts[ib]);

    /* Run the regularized inversion */
    conj_grad *cg = new conj_grad(_pefniter,false);
    cg->run(dk,iop,idat,zro,flts[ib],_eps);
    lvf->set_zerolag(flts[ib]);
    pefops.push_back(new lv_conv3dm(_nd,_j,_nlag,flts[ib]));
    /* Clean up */
    delete ssht; delete cg; delete zro;
  }
  delete psht; delete idat;
}

void mcpefdblnd::apply(hypercube_float *bdata, hypercube_float *inddata, hypercube_float *dbdata) {

  /* Create shot container */
  int k = 0;
  std::vector<std::vector<hypercube_float*>> shts;
  for(int is = 0; is < (int)_nb.size(); ++is) {
    std::vector<hypercube_float*> isht;
    for(int ib = 0; ib < _nb[is]; ++ib) {
      hypercube_float *tsht = new hypercube_float(_saxes,true);
      memcpy(tsht->vals,&inddata->vals[k*_nt*_nrx*_nc],sizeof(float)*_nt*_nrx*_nc);
      isht.push_back(tsht);
      k++;
    }
    shts.push_back(isht);
  }

  /* Loop over all shots */
  omp_set_num_threads(_nthrds);
#pragma omp parallel for default(shared)
  for(int is = 0; is < (int)_nb.size(); ++is) {
    fprintf(stderr,"blended shot %d\n",is);
    hypercube_float *bldsht = new hypercube_float(_saxes,true);
    std::vector<hypercube_float*> isht = shts[is];
    std::vector<lv_conv3dm*> pefops;
    /* Check if shot is blended */
    if(_nb[is] < 2) {
      std::vector<hypercube_float*> outs;
      memcpy(bldsht->vals,&bdata->vals[is*_nt*_nrx*_nc],sizeof(float)*_nt*_nrx*_nc);
      outs.push_back(bldsht);
      _odatcont[is] = outs;
      continue;
    }
    /* Estimate a PEF for each independent shot */
    find_pefs(is, isht, pefops);

    /* Signal and noise inversion */
    fprintf(stderr,"blended shot %d signal and noise inversion.\n",is);
    /* Get scaled and padded blended shot */
    memcpy(bldsht->vals,&bdata->vals[is*_nt*_nrx*_nc],sizeof(float)*_nt*_nrx*_nc);
    hypercube_float *pbldsht = new hypercube_float(_paxes,true);
    _pad.forward(false,bldsht,pbldsht);
    pefsignoi3d *A = new pefsignoi3d(pbldsht,pefops[0],pefops[1]);

    /* Create weighted data */
    hypercube_float *wdats = new hypercube_float(_paxes,true);
    hypercube_float *wdatn = new hypercube_float(_paxes,true);
    wdats->zero(); wdatn->zero();
    pefops[0]->forward(false,pbldsht,wdats);
    pefops[1]->forward(false,pbldsht,wdatn);

    /* Total data vector */
    super_vector *wdat   = new super_vector(wdatn,wdats);
    super_vector *datinv = new super_vector(pbldsht,wdat);

    /* Multi-model vector */
    hypercube_float *sig  = new hypercube_float(_paxes,true);
    hypercube_float *noiz = new hypercube_float(_paxes,true);
    sig->zero(); noiz->zero();
    super_vector *sn = new super_vector(sig,noiz);

    conj_grad cg = conj_grad(_sniniter);
    //if(!_smmvtags[is].empty()) {
    //  cg.set_mod1out("smmv"+std::to_string(is));
    //}
    //if(!_nmmvtags[is].empty()) {
    //  cg.set_mod2out("nmmv"+std::to_string(is));
    //}
    cg.run(A,datinv,sn);

    /* Save output */
    std::vector<hypercube_float*> outs;
    outs.push_back((hypercube_float*)sn->vecs[0]);
    outs.push_back((hypercube_float*)sn->vecs[1]);
    _odatcont[is] = outs;

    /* Clean up */
    delete bldsht; delete pbldsht;
    delete wdats; delete wdatn;
    delete wdat; delete datinv;
    delete pefops[0]; delete pefops[1];
  }

  k = 0;
  for(int is = 0; is < (int)_nb.size(); ++is) {
    std::vector<hypercube_float*> odat = _odatcont[is];
    for(int ib = 0; ib < _nb[is]; ++ib) {
      hypercube_float *tdat = new hypercube_float(_saxes,true);
      tdat->zero();
      _pad.adjoint(false,tdat,odat[ib]);
      memcpy(&dbdata->vals[k*_nt*_nrx*_nc],tdat->vals,sizeof(float)*_nt*_nrx*_nc);
      k++;
    }
  }

}
