/**
 * Deblends multicomponent data using
 * multicomponent PEFs estimated
 * on the indpenednetly modeled data
 * @author: Joseph Jennings
 * @version: 2017.05.01
 */

#ifndef MCPEFDBLND_H_
#define MCPEFDBLND_H_

#include <vector>
#include "hypercube_float.h"
#include "lv_conv3dmask.h"
#include "lv_conv3dm.h"
#include "pefsignoi3d.h"
#include "pad3d.h"

class mcpefdblnd {
  public:
    mcpefdblnd(){};
    mcpefdblnd(int pefniter, int sniniter, float eps, std::vector<int> j,
        std::vector<int> nlag, float shtscale, int minblnd, int maxlbnd, int nsx,
        std::vector<axis> daxes, int nthrds);
    void set_smmvtags(std::vector<std::string> smmvtags);
    void set_nmmvtags(std::vector<std::string> nmmvtags);
    void find_pefs(int is, std::vector<hypercube_float*> iisht, std::vector<lv_conv3dm*> &pefops);
    void apply(hypercube_float *bdata, hypercube_float *inddata, hypercube_float *dbdata);
    void set_padding(int ntp, int nrxp, int ncp);
    ~mcpefdblnd(){};
  private:
    int _pefniter, _sniniter, _nt, _nrx, _nc, _ntp, _nrxp, _ncp, _nthrds;
    float _eps, _shtscale;
    float _dt, _drx, _dc, _ot, _orx, _oc;
    std::vector<int> _nb, _nd, _j, _nlag;
    std::vector<axis> _saxes, _paxes;
    std::vector<std::string> _smmvtags, _nmmvtags;
    std::vector<std::vector<hypercube_float*>> _odatcont;
    pad3d _pad;
};

#endif /* MCPEFDBLND_H_ */
