/*
 * pad3d.cpp
 *
 *  Created on: Apr 29, 2017
 *      Author: joe
 */

#include "pad3d.h"
#include "hypercube_float.h"

pad3d::pad3d(int i1loc, int i2loc, int i3loc, float fill) {
  _i1loc = i1loc; _i2loc = i2loc; _i3loc = i3loc; _fill = fill;
}

bool pad3d::forward(bool add, my_vector *mod, my_vector *dat) {
  dat->set_val(_fill);

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int n1p = d->get_axis(1).n; int n2p = d->get_axis(2).n; int n3p = d->get_axis(3).n;
  int n1  = m->get_axis(1).n; int n2  = m->get_axis(2).n; int n3  = m->get_axis(3).n;

  if(n1p < n1 || n2p < n2 || n3p < n3) {
    seperr("pad3d: Output padded image will be smaller than input.\n");
  }
  if(_i1loc > n1 || _i2loc > n2 || _i3loc > n3) {
    seperr("pad3d: initial location is out of bounds.\n");
  }

  for(int i3 = 0; i3 < n3; ++i3) {
    for(int i2 = 0; i2 < n2; ++i2) {
      for(int i1 = 0; i1 < n1; ++i1) {
        d->vals[(i3+_i3loc)*n1p*n2p + (i2+_i2loc)*n1p + i1+_i1loc] = m->vals[i3*n1*n2 + i2*n1 + i1];
      }
    }
  }

  return true;
}

bool pad3d::adjoint(bool add, my_vector *mod, my_vector *dat) {

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int n1p = d->get_axis(1).n; int n2p = d->get_axis(2).n; int n3p = d->get_axis(3).n;
  int n1  = m->get_axis(1).n; int n2  = m->get_axis(2).n; int n3  = m->get_axis(3).n;

  if(n1p < n1 || n2p < n2 || n3p < n3) {
    seperr("pad3d: Output padded image will be smaller than input.\n");
  }
  if(_i1loc > n1 || _i2loc > n2 || _i3loc > n3) {
    seperr("pad3d: initial location is out of bounds.\n");
  }

  for(int i3 = 0; i3 < n3; ++i3) {
    for(int i2 = 0; i2 < n2; ++i2) {
      for(int i1 = 0; i1 < n1; ++i1) {
        m->vals[i3*n1*n2 + i2*n1 + i1] = d->vals[(i3+_i3loc)*n1p*n2p + (i2+_i2loc)*n1p + i1+_i1loc];

      }
    }
  }

  return true;
}
