/**
 * Separates signal and noise using
 * 3D prediction-error filters
 * Am = [ I  I ][ s ]
 *      [ N  0 ][ n ]
 *      [ 0  S ]
 * where I are identity operators,
 * N is a noise PEF and S is a signal PEF.
 * @author: Joseph Jennings
 * @version: 2017.04.29
 */

#ifndef PEFSIGNOI3D_H_
#define PEFSIGNOI3D_H_

#include "combo_oper.h"
#include "my_operator.h"
#include "lv_conv3dm.h"

class pefsignoi3d : public my_operator {
  public:
    pefsignoi3d(){};
    pefsignoi3d(hypercube_float *dat, lv_conv3dm* spef, lv_conv3dm* npef);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    ~pefsignoi3d(){};

  private:
    row_op *_gam;
    diag_op *_pefs;
    col_op *_A;
};

#endif /* PEFSIGNOI3D_H_ */
