/*
 * pvrrec.cpp
 *
 *  Created on: Apr 24, 2017
 *      Author: joe
 */

#include "pvrrec.h"
#include "simpleplot.h"

pvrrec::pvrrec(std::vector<axis> uaxes, axis arx, int recz, float rho) {
  if(uaxes.size() != 3) seperr("Error: pvrrec must operate on 2D wavefields.\n");
  _nz = uaxes[0].n; _nx = uaxes[1].n; _nt = uaxes[2].n; _nrx = arx.n;
  /* Temporary wavefield and shot arrays */
  _wfld  = new hypercube_float(uaxes,true);
  _wfldv = new hypercube_float(uaxes,true);
  std::vector<axis> shaxes, mcaxes;
  mcaxes = uaxes; mcaxes.push_back(axis(2,0.0,1.0));
  _mcwfld = new hypercube_float(mcaxes,true);
  shaxes.push_back(uaxes[2]); shaxes.push_back(arx);
  _sht = new hypercube_float(shaxes,true);
  /* Receiver restriction and grad operators */
  _rr   = rrec(arx,recz);
  _dzdx = igrad2cent10th(uaxes);
  _iw   = new integratewfld(uaxes,rho);
}

bool pvrrec::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Build the multicomponent wavefield */
  for(int ic = 0; ic < 2; ++ic) {
    memcpy(_sht->vals,&m->vals[_nt*_nrx*(ic)],sizeof(float)*_nt*_nrx);
    /* Inject the component */
    _rr.forward(false,_sht,_wfld);
    /* Integrate the wavefield */
    _iw->adjoint(false,_wfldv,_wfld);
    memcpy(&_mcwfld->vals[(ic)*_nt*_nx*_nz],_wfldv->vals,sizeof(float)*_nt*_nx*_nz);
  }
  _dzdx.adjoint(false,d,_mcwfld);

  return true;
}

bool pvrrec::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  /* Compute the pressure gradient */
  _dzdx.forward(false,d,_mcwfld);
  for(int ic = 0; ic < 2; ++ic) {
    /* Extract each component of the pressure gradient */
    memcpy(_wfld->vals,&_mcwfld->vals[_nt*_nx*_nz*ic],sizeof(float)*_nt*_nx*_nz);
    /* Integrate each component for the particle velocity */
    _iw->forward(false,_wfld,_wfldv);
    _rr.adjoint(false,_sht,_wfldv);
    memcpy(&m->vals[(ic)*_nt*_nrx],_sht->vals,sizeof(float)*_nt*_nrx);
  }
  return true;
}
