/**
 * Particle velocity receiver
 * restriction operator. Essentially
 * places a mask over the pressure.
 * @author: Joseph Jennings
 * @version: 2017.04.24
 */
#ifndef PVRREC_H_
#define PVRREC_H_

#include "my_operator.h"
#include "hypercube_float.h"
#include "rrec.h"
#include "igrad2.h"
#include "igrad2cent10th.h"
#include "integratewfld.h"

class pvrrec : public my_operator {
  public:
    pvrrec(){};
    pvrrec(std::vector<axis> uaxes, axis arx, int recz, float rho);
    virtual bool forward(bool add, my_vector *mod, my_vector *dat);
    virtual bool adjoint(bool add, my_vector *mod, my_vector *dat);
    ~pvrrec(){
      delete _mcwfld; delete _wfld; delete _wfldv; delete _sht;
      delete _iw;
    };

  private:
    int _nx, _nz, _nt, _nrx;
    hypercube_float *_mcwfld, *_wfld, *_wfldv, *_sht;
    rrec _rr;
    igrad2cent10th _dzdx;
    integratewfld *_iw;
};

#endif /* PVRREC_H_ */
