/**
 * Creates blended data that is of
 * the correct format for imaging
 * with an independent modeling
 * operator
 * @author: Joseph Jennings
 * @version: 2017.05.05
 */

#include "sep_params.h"
#include "../main/blendnc2d.h"
#include "../main/simpleplot.h"

int main(int argc, char ** argv) {

  sep_params pars = sep_params(argc,argv);
  std::vector<int> maxblnd  = pars.get_auxvints("maxblnd",1,"in");
  std::vector<int> minblnd  = pars.get_auxvints("maxblnd",1,"in");
  std::vector<float> xss    = pars.get_auxvfloats("xss",100,"in");

  int nsx = (int)xss.size();
  /* Build blending array */
  blendnc2d *gam = new blendnc2d(nsx,minblnd[0],maxblnd[0]);
  std::vector<int> nb = gam->get_nb();

  sregf bldin = sregf("in");

  int nt  = bldin.get_axis(1).n;
  int nrx = bldin.get_axis(2).n;
  int nc  = bldin.get_axis(3).n;

  axis saxis = axis(nsx,0.0,1.0);
  std::vector<axis> caxes, shaxes;
  caxes.push_back(bldin.get_axis(1));
  caxes.push_back(bldin.get_axis(2));
  caxes.push_back(bldin.get_axis(3));
  caxes.push_back(saxis);
  shaxes.push_back(bldin.get_axis(1));
  shaxes.push_back(bldin.get_axis(2));
  shaxes.push_back(bldin.get_axis(3));

  hypercube_float *blddat = new hypercube_float(bldin.get_axes(),true);
  hypercube_float *cbddat = new hypercube_float(caxes,true);
  hypercube_float *bldsht = new hypercube_float(shaxes,true);

  bldin.read_all("in",blddat);

  int j = 0;
  for(int is = 0; is < (int)nb.size(); ++is) {
    memcpy(bldsht->vals,&blddat->vals[is*nt*nrx*nc],sizeof(float)*nt*nrx*nc);
    for(int ib = 0; ib < nb[is]; ++ib) {
      memcpy(&cbddat->vals[j*nt*nrx*nc],bldsht->vals,sizeof(float)*nt*nrx*nc);
      ++j;
    }
  }

  /* Write out files */
  sregf cout = sregf(cbddat->get_axes());
  cout.write_description("out");
  cout.write_all("out",cbddat);
}
