/**
 * Creates a velocity model mask for
 * creating different geological
 * scenarios
 * @author: Joseph Jennings
 * @version: 2017.05.14
 */

#include <math.h>
#include "sep_params.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  /* Spatial axes */
  int nz   = pars.get_int("n1",751);
  float oz = pars.get_float("o1",0.0);
  float dz = pars.get_float("d1",4.0);
  int nx   = pars.get_int("n2",2301);
  float ox = pars.get_float("o2",0.0);
  float dx = pars.get_float("d2",4.0);
  /* Syncline parameters */
  float sigma = pars.get_float("sigma",900.0);
  float center = pars.get_float("center",4750.0);
  float scale = pars.get_float("scale",500.0);
  float zl = pars.get_float("z1",1500.0);
  float v1 = pars.get_float("v1",2000.0);
  float v2 = pars.get_float("v2",2500.0);

  std::vector<axis> axx, vaxes;
  vaxes.push_back(axis(nz,oz,dz));
  vaxes.push_back(axis(nx,ox,dx));
  axx.push_back(axis(nx,ox,dx));

  hypercube_float *nuvel = new hypercube_float(vaxes,true);
  hypercube_float *bndx = new hypercube_float(axx,true);


  float denom = (sigma*sigma);
  for(int i = 0; i < nx; ++i) {
    float x = ox + i*dx;
    float arg = (x-center);
    bndx->vals[i] = zl + scale*exp(-arg*arg/denom);
  }

  nuvel->set_val(v1);
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      float z = oz + iz*dz;
      if( z > bndx->vals[ix]) {
        nuvel->vals[ix*nz + iz] = v2;
      }
    }
  }

  /* Write out velocity */
  sregf velout = sregf(nuvel->get_axes());
  velout.write_description("out");
  velout.write_all("out",nuvel);

  /* Clean up */
  delete nuvel; delete bndx;
}
