Program alibrn

  use sep
  use lint_mod
  use sscale_mod
  use step_mod
  use scats0_mod
  use secder_mod
  use mem_mod

  implicit none

  integer                             :: nx,nz,nt,ntw,nrx
  integer                             :: sx,sz,wfldj,irec
  integer                             :: bc1,bc2,bc3,bc4
  real                                :: ox,oz,ot,otw
  real                                :: dx,dz,dt,dtw
  real                                :: rec_z,rec_x,orx,drx
  real                                :: fmax
  real, dimension(:,:),   allocatable :: wt,vwt,vel,veldt2,dm,dd
  real, dimension(:,:,:), allocatable :: u0, u0sub, temp1, temp2, ssecf, bwfld
  logical                             :: adj 

  call sep_init()

  ! Get axes from history files

  call from_param("nt",nt,200)
  call from_param("dt",dt,0.004)

  call from_aux("u0","d3",dtw)
  call from_aux("u0","n3",ntw)

  ! Velocity parameters
  call from_aux("vel","n1",nx)
  call from_aux("vel","o1",ox)
  call from_aux("vel","d1",dx)
  call from_aux("vel","n2",nz)
  call from_aux("vel","o2",oz)
  call from_aux("vel","d2",dz)

  call from_param("adj",adj,.false.)
  call from_param("bc1",bc1,50)
  call from_param("bc2",bc2,50)
  call from_param("bc3",bc3,50)
  call from_param("bc4",bc4,50)
  call from_param("ntw",ntw,800)
  call from_param("wfldj",wfldj,4)
  call from_param("nrx",nrx,200)
  call from_param("orx",orx,52.0)
  call from_param("drx",drx,1.0)
  call from_param("recz",rec_z,151.0)

  ! Allocate memory
  call mem_alloc2d(vel,nz,nx)
  call mem_alloc2d(veldt2,nz,nx)
  call mem_alloc2d(wt,nz,nx)
  call mem_alloc2d(dm,nz,nx)
  call mem_alloc2d(vwt,nz,nx)
  call mem_alloc2d(dd,nt,nrx)
  call mem_alloc3d(u0,nz,nx,ntw)
  call mem_alloc3d(u0sub,nz,nx,nt)
  call mem_alloc3d(temp1,nz,nx,nt)
  call mem_alloc3d(temp2,nz,nx,nt)
  call mem_alloc3d(ssecf,nz,nx,ntw)
  call mem_alloc3d(bwfld,nz,nx,ntw)

  ! Read in files
  if(adj) then
    call sreed("in",dd,nt*nrx*4)
  else
    call sreed("in",dm,nz*nx*4)
  endif

  call sreed("vel",vel,nx*nz*4)
  call sreed("wt",wt,nx*nz*4)
  call sreed("vwt",vwt,nx*nz*4)
  call sreed("u0",u0,nx*nz*ntw*4)

  veldt2 = vel*vel*dtw*dtw

  ! Write out ABC weight
  call auxputch("n1","d",nz,"wtout")
  call auxputch("o1","f",oz,"wtout")
  call auxputch("d1","f",dz,"wtout")
  call auxputch("n2","d",nx,"wtout")
  call auxputch("o2","f",ox,"wtout")
  call auxputch("d2","f",dx,"wtout")
  call srite("wtout",wt,nx*nz*4)

  ! Write out vwt
  call auxputch("n1","d",nz,"vwtout")
  call auxputch("o1","f",oz,"vwtout")
  call auxputch("d1","f",dz,"vwtout")
  call auxputch("n2","d",nx,"vwtout")
  call auxputch("o2","f",ox,"vwtout")
  call auxputch("d2","f",dx,"vwtout")
  call srite("vwtout",vwt,nx*nz*4)

  ! Initialize operators
  call sscale_init(nz, nx, nt, dtw, veldt2, wt)
  call lint_init(nz,nx,nt,wfldj)
  call step_init(nz, nx, ntw, bc1, bc2, bc3, bc4, dz, dx, veldt2, vwt, wt)

  call lint_op2(.true.,.false.,u0sub,u0)
  call scats0_init(nz,nx,nt,u0sub)
  call secder_init(nx,nz,nt,dt)

  if(adj) then
    dm = 0
    temp1 = 0.
    do irec=1,nrx
      rec_x = (irec-1)*drx + orx
      temp1(rec_z,rec_x,:) = temp1(rec_z,rec_x,:) + dd(:,irec)
    end do
    call lint_op2(.false.,.false.,temp1,ssecf)
    call step_op2(.true.,.false.,bwfld,ssecf)
    call lint_op2(.true.,.false.,temp2,bwfld)
    call sscale_op2(.true.,.false.,temp1,temp2)
    call secder_op2(.true.,.false.,temp2,temp1)
    call scats0_op2(.true.,.true.,dm,temp2)
  else
    call scats0_op2(.false.,.false.,dm,temp1)
    call secder_op2(.false.,.false.,temp1,temp2)
    call sscale_op2(.false.,.false.,temp2,temp1)
    call lint_op2(.false.,.false.,temp1,ssecf)
    call step_op2(.false.,.false.,ssecf,bwfld)
  endif

  if(adj) then
    call putch("n1","d",nz);  call putch("o1","f",oz); call putch("d1","f",dz)
    call putch("n2","d",nx);  call putch("o2","f",ox); call putch("d2","f",dx)
    !call putch("n3","d",ntw); call putch("o3","f",ot); call putch("d3","f",dtw)
    call putch("n3","d",1); call putch("o3","f",0.0); call putch("d3","f",1.0)
    call srite("out",dm,nx*nz*4)
    !call srite("out",bwfld,nx*nz*ntw*4)
  else
    call putch("n1","d",nz);  call putch("o1","f",oz); call putch("d1","f",dz)
    call putch("n2","d",nx);  call putch("o2","f",ox); call putch("d2","f",dx)
    call putch("n3","d",ntw); call putch("o3","f",ot); call putch("d3","f",dtw)
    !call putch("n3","d",nt); call putch("o3","f",ot); call putch("d3","f",dt)
    call srite("out",bwfld,nx*nz*ntw*4)
    !call srite("out",u0sub,nx*nz*nt*4)
  endif

end program
