Program myex

  use sep
  use lint_mod
  use sscale_mod
  use step_mod
  use tomo_mod
  use mem_mod

  implicit none

  integer                             :: nx,nz,nt,ntw
  integer                             :: sx,sz,wfldj
  integer                             :: bc1,bc2,bc3,bc4
  real                                :: ox,oz,ot,otw
  real                                :: dx,dz,dt,dtw
  real                                :: fmax
  real, dimension(:),     allocatable :: src
  real, dimension(:,:),   allocatable :: wt,vwt,vel,veldt2
  real, dimension(:,:,:), allocatable :: srcp, srcps, srcpsi, wfd
  logical                             :: adj

  call sep_init()

  ! Get axes from history files
  call hetch("n1","d",nt)
  call hetch("o1","f",ot)
  call hetch("d1","f",dt)

  call from_aux("vel","n1",nx)
  call from_aux("vel","o1",ox)
  call from_aux("vel","d1",dx)
  call from_aux("vel","n2",nz)
  call from_aux("vel","o2",oz)
  call from_aux("vel","d2",dz)

  call from_param("adj",adj,.false.)
  call from_param("sx",sx,150)
  call from_param("sz",sz,150)
  call from_param("bc1",bc1,50)
  call from_param("bc2",bc2,50)
  call from_param("bc3",bc3,50)
  call from_param("bc4",bc4,50)
  call from_param("ntw",ntw,800)
  call from_param("dtw",dtw,0.001)
  call from_param("wfldj",wfldj,4)

  ! Allocate memory
  call mem_alloc1d(src,nt)
  call mem_alloc2d(vel,nz,nx)
  call mem_alloc2d(veldt2,nz,nx)
  call mem_alloc2d(wt,nz,nx)
  call mem_alloc2d(vwt,nz,nx)
  call mem_alloc3d(srcp,nz,nx,nt)
  call mem_alloc3d(srcps,nz,nx,nt)
  call mem_alloc3d(srcpsi,nz,nx,ntw)
  call mem_alloc3d(wfd,nz,nx,ntw)

  ! Read in files
  if(adj) then
    call sreed("in",wfd,nz*nx*ntw*4)
  else
    call sreed("in",src,nt*4)
  endif

  call sreed("vel",vel,nx*nz*4)
  call sreed("wt",wt,nx*nz*4)
  call sreed("vwt",vwt,nx*nz*4)

  veldt2 = vel*vel*dtw*dtw

  !call AbcWt(vel, vwt, wt, dtw, dz, dx, bc1, bc2, bc3, bc4)

  ! Write out ABC weight
  call auxputch("n1","d",nz,"wtout")
  call auxputch("o1","f",oz,"wtout")
  call auxputch("d1","f",dz,"wtout")
  call auxputch("n2","d",nx,"wtout")
  call auxputch("o2","f",ox,"wtout")
  call auxputch("d2","f",dx,"wtout")
  call srite("wtout",wt,nx*nz*4)

  ! Write out vwt
  call auxputch("n1","d",nz,"vwtout")
  call auxputch("o1","f",oz,"vwtout")
  call auxputch("d1","f",dz,"vwtout")
  call auxputch("n2","d",nx,"vwtout")
  call auxputch("o2","f",ox,"vwtout")
  call auxputch("d2","f",dx,"vwtout")
  call srite("vwtout",vwt,nx*nz*4)

  ! Initialize operators
  srcp(sz+1,sx+1,:) = src
  call sscale_init(nz, nx, nt, dtw, veldt2, wt)
  call lint_init(nz,nx,nt,wfldj)
  call step_init(nz, nx, ntw, bc1, bc2, bc3, bc4, dz, dx, veldt2, vwt, wt)

  if(adj) then
    call step_op2(.true.,.false.,srcpsi,wfd)
  else
    call sscale_op2(.false.,.false.,srcp,srcps)
    call lint_op2(.false., .false., srcps, srcpsi)
    call step_op2(.false.,.false.,srcpsi,wfd)
  endif

  call putch("n1","d",nz);  call putch("o1","f",oz); call putch("d1","f",dz)
  call putch("n2","d",nx);  call putch("o2","f",ox); call putch("d2","f",dx)
  call putch("n3","d",ntw); call putch("o3","f",ot); call putch("d3","f",dtw)
  if(adj) then
    call srite("out",srcpsi,nx*nz*ntw*4)
  else
    call srite("out",wfd,nx*nz*ntw*4)
  endif

  deallocate(src,wfd,srcp,srcpsi)

end program myex
