/**
 * Performs the dot-product test of the
 * boundary derivative operator
 * @author: Joseph Jennings
 * @version: 2016.11.27
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/bcder.h"
#include "../main/zeropadimg.h"
#include "../main/simpleplot.h"
#include "hypercube_float.h"

class bcdertest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(bcdertest,dptest){
  int nx = 100; int nz = 100;
  int p = 4;
  /* Build axes */
  axis ax = axis(nx,  0.0, 1.0);
  axis az = axis(nz,  0.0, 1.0);
  axis px = axis(nx+p,  0.0, 1.0);
  axis pz = axis(nz+p,  0.0, 1.0);
  std::vector<axis> axes;
  axes.push_back(az); axes.push_back(ax);
  std::vector<axis> paxes;
  paxes.push_back(pz); paxes.push_back(px);
  /* Build hypercubes */
  hypercube_float *rm = new hypercube_float(axes, true);
  hypercube_float *rd = new hypercube_float(axes, true);
  hypercube_float *m  = new hypercube_float(paxes, true);
  hypercube_float *ms = new hypercube_float(paxes, true);
  hypercube_float *d  = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  std::vector<int> nbs;
  int bz1=20; int bz2=20; int bx1=20; int bx2=20;
  nbs.push_back(bz1); nbs.push_back(bx1);
  nbs.push_back(bz2); nbs.push_back(bx2);
  rm->random(); rd->random();
  bcder bd = bcder(paxes, nbs, p);
  zeropadimg zp = zeropadimg(p/2,p/2,0.0);
  zp.forward(false,rm,m); zp.forward(false,rd,d);
  /* Add == false */
  bd.forward(false, m, ds);
  float dotd = d->dot(ds);
  bd.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  /* Add == true */
  bd.forward(true,m,ds);
  float adotd = d->dot(ds);
  bd.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.000001);
  delete m; delete ms;
  delete d; delete ds;
}
