/**
 * Tests the adjoint of the Born
 * scattering operator
 * @author: Joseph Jennings
 * @version: 2016.12.28
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/bscat.h"
#include "axis.h"
#include "hypercube_float.h"
#include "../main/simpleplot.h"

class bscattest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(bscattest,dptest){
  int nx = 100; int nz = 100;
  int nts = 250;
  int xs = 50; int zs = 0;
  /* Build axes */
  axis ax = axis(nx,  0.0, 1.0);
  axis az = axis(nz,  0.0, 1.0);
  axis at = axis(nts, 0.0, 1.0);
  std::vector<axis> vaxes, waxes;
  waxes.push_back(az);
  waxes.push_back(ax); waxes.push_back(at);
  vaxes.push_back(az); vaxes.push_back(ax);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(vaxes, true);
  hypercube_float *ms = new hypercube_float(vaxes, true);
  hypercube_float *d  = new hypercube_float(waxes, true);
  hypercube_float *ds = new hypercube_float(waxes, true);
  hypercube_float *u0 = new hypercube_float(waxes, true);
  m->random(); d->random(); u0->random();
  bscat brn = bscat(u0);
  /* Add == false */
  brn.forward(false, m, ds);
  float dotd = d->dot(ds);
  brn.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.0001);
  /* Add == true */
  brn.forward(true,m,ds);
  float adotd = d->dot(ds);
  brn.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.0001);
  delete m; delete ms;
  delete d; delete ds;
}
