/**
 * Performs the dot-product test
 * of the no-boundary finite difference
 * coefficient operator
 * @author: Joseph Jennings
 * @version: 2016.12.7
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/fdcoefnb.h"
#include "../main/zeropadimg.h"
#include "../main/laplacian4th.h"
#include "../main/laplacian10th.h"
#include "../main/simpleplot.h"
#include "hypercube_float.h"

class fdcoefnbtest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

/* Fourth order laplacian test */
TEST_F(fdcoefnbtest,dptest4){
  int nx = 100; int nz = 100;
  int nt = 10;
  int p = 4;
  float dt2 = (0.001*0.001);
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  axis at  = axis(nt  , 0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az); axes.push_back(ax);
  axes.push_back(at);
  paxes.push_back(azp); paxes.push_back(axp);
  paxes.push_back(at);
  /* Build hypercubes */
  hypercube_float *rm  = new hypercube_float(axes , true);
  hypercube_float *rd  = new hypercube_float(axes , true);
  hypercube_float *m = new hypercube_float(paxes, true);
  hypercube_float *ms = new hypercube_float(paxes, true);
  hypercube_float *d = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  hypercube_float *v  = new hypercube_float(paxes, true);
  /* Initialize hypercubes */
  rm->random(); rd->random();
  v->set_val(3000); v->mult(v); v->scale(dt2);
  zeropadimg pad = zeropadimg(2,2,0.0);
  laplacian4th *lap = new laplacian4th(paxes);
  fdcoefnb coef = fdcoefnb(paxes,lap,v);
  /* Pad the wavefield (for the laplacian) */
  pad.forward(false,rd,d); pad.forward(false,rm,m);
  /* Add == false */
  coef.forward(false,m,ds);
  float dotd = d->dot(ds);
  coef.adjoint(false,ms,d);
  float dotm = m->dot(ms);
  /* Decreased precision for running all tests */
  EXPECT_NEAR(dotd, dotm, 0.0001);
  delete m; delete ms;
  delete d; delete ds;
  delete rm; delete rd;
  delete v;
}

/* Tenth order laplacian test */
TEST_F(fdcoefnbtest,dptest10){
  int nx = 100; int nz = 100;
  int nt = 10;
  int p = 10;
  float dt2 = (0.001*0.001);
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  axis at  = axis(nt  , 0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az); axes.push_back(ax);
  axes.push_back(at);
  paxes.push_back(azp); paxes.push_back(axp);
  paxes.push_back(at);
  /* Build hypercubes */
  hypercube_float *rm  = new hypercube_float(axes , true);
  hypercube_float *rd  = new hypercube_float(axes , true);
  hypercube_float *m = new hypercube_float(paxes, true);
  hypercube_float *ms = new hypercube_float(paxes, true);
  hypercube_float *d = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  hypercube_float *v  = new hypercube_float(paxes, true);
  /* Initialize hypercubes */
  rm->random(); rd->random();
  v->set_val(3000); v->mult(v); v->scale(dt2);
  zeropadimg pad = zeropadimg(5,5,0.0);
  laplacian10th *lap = new laplacian10th(paxes);
  fdcoefnb coef = fdcoefnb(paxes,lap,v);
  /* Pad the wavefield (for the laplacian) */
  pad.forward(false,rd,d); pad.forward(false,rm,m);
  /* Add == false */
  coef.forward(false,m,ds);
  float dotd = d->dot(ds);
  coef.adjoint(false,ms,d);
  float dotm = m->dot(ms);
  /* Decreased precision for running all tests */
  EXPECT_NEAR(dotd, dotm, 0.0001);
  delete m; delete ms;
  delete d; delete ds;
  delete rm; delete rd;
  delete v;
}
