/**
 * Tests the dot product
 * of the laplacian operator
 * @author: Joseph Jennings
 * @version: 2016.11.25
 */
#include <limits.h>
#include "gtest/gtest.h"
#include "../main/laplacian4th.h"
#include "../main/laplacian10th.h"
#include "axis.h"
#include "hypercube_float.h"

//TODO: Add another test for testing the approximation of
//      the derivative
class laplaciantest: public ::testing::Test {
  public:
    void trunc(int loc1, int loc2, hypercube_float *dat, hypercube_float* pad);
    void pad(int loc1, int loc2, hypercube_float *dat, hypercube_float* pad);
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(laplaciantest,dptest4){
  int nx = 100; int nz = 100;
  int p = 4;
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az); axes.push_back(ax);
  paxes.push_back(azp); paxes.push_back(axp);
  /* Build hypercubes */
  hypercube_float *rm = new hypercube_float(axes, true);
  hypercube_float *rd = new hypercube_float(axes, true);
  hypercube_float *m  = new hypercube_float(paxes , true);
  hypercube_float *ms = new hypercube_float(paxes, true);
  hypercube_float *d  = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  rm->random(); rd->random();
  pad(2,2,rm,m); pad(2,2,rd,d);
  laplacian4th lap = laplacian4th(paxes);
  /* Add == false */
  lap.forward(false, m, ds);
  float dotd = d->dot(ds);
  lap.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  /* Add == true */
  lap.forward(true, m, ds);
  float adotd = d->dot(ds);
  lap.adjoint(true, ms, d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.000001);
  delete m; delete ms;
  delete d; delete ds;
}

TEST_F(laplaciantest,dptest10){
  int nx = 100; int nz = 100;
  int p = 10;
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az); axes.push_back(ax);
  paxes.push_back(azp); paxes.push_back(axp);
  /* Build hypercubes */
  hypercube_float *rm = new hypercube_float(axes, true);
  hypercube_float *rd = new hypercube_float(axes, true);
  hypercube_float *m  = new hypercube_float(paxes, true);
  hypercube_float *ms = new hypercube_float(paxes, true);
  hypercube_float *d  = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  rm->random(); rd->random();
  pad(5,5,rm,m); pad(5,5,rd,d);
  laplacian10th lap = laplacian10th(paxes);
  /* Add == false */
  lap.forward(false, m, ds);
  float dotd = d->dot(ds);
  lap.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  /* Add == true */
  lap.forward(true, m, ds);
  float adotd = d->dot(ds);
  lap.adjoint(true, ms, d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.000001);
  delete m; delete ms;
  delete d; delete ds;
}

/**
 * Pads a 2D image
 * @param loc1 the offset along axis 1
 * @param loc2 the offset along axis 1
 * @param dat  the input data
 * @param pad  the padded result
 */
void laplaciantest::pad(int loc1, int loc2, hypercube_float *dat, hypercube_float* pad) {
  pad->zero();
  int n1p = pad->get_axis(1).n; int n2p = pad->get_axis(2).n;
  int n1  = dat->get_axis(1).n; int n2  = dat->get_axis(2).n;
  for(int i2 = 0; i2 < n2; ++i2) {
    for(int i1 = 0; i1 < n1; ++i1) {
      pad->vals[(i2+loc2)*n1p + i1 + loc1] = dat->vals[i2*n1 + i1];
    }
  }
}

/**
 * Truncates a 2D image (the adjoint of Pad)
 * @param loc1 the offset along axis 1
 * @param loc2 the offset along axis 1
 * @param dat  the truncated result
 * @param pad  the input padded data
 */
void laplaciantest::trunc(int loc1, int loc2, hypercube_float *dat, hypercube_float* pad) {
  int n1p = pad->get_axis(1).n; int n2p = pad->get_axis(2).n;
  int n1  = dat->get_axis(1).n; int n2  = dat->get_axis(2).n;
  for(int i2 = 0; i2 < n2; ++i2) {
    for(int i1 = 0; i1 < n1; ++i1) {
      dat->vals[i2*n1 + i1] = pad->vals[(i2+loc2)*n1p + i1+loc1];
    }
  }
}
