/**
 * Tests the adjoint of the
 * receiver restriction operator
 * @author: Joseph Jennings
 * @version: 2016.12.27
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/rrec.h"
#include "axis.h"
#include "hypercube_float.h"

class rrectest : public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(rrectest,dptest){
  int nx = 300;  int nz = 300;
  int nts = 250; int nrx = 200;
  float orx = 51.0; float drx = 1.0;
  float recz = 150;
  axis recx = axis(nrx, orx, drx);
  /* Build axes */
  axis ax = axis(nx,  0.0, 1.0);
  axis az = axis(nz,  0.0, 1.0);
  axis at = axis(nts, 0.0, 1.0);
  std::vector<axis> waxes, daxes;
  waxes.push_back(az);
  waxes.push_back(ax); waxes.push_back(at);
  daxes.push_back(at); daxes.push_back(recx);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(waxes, true);
  hypercube_float *ms = new hypercube_float(waxes, true);
  hypercube_float *d  = new hypercube_float(daxes, true);
  hypercube_float *ds = new hypercube_float(daxes, true);
  m->random(); d->random();
  rrec rr = rrec(recx, recz);
  /* Add == false */
  rr.forward(false, m, ds);
  float dotd = d->dot(ds);
  rr.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  /* Add == true */
  rr.forward(true,m,ds);
  float adotd = d->dot(ds);
  rr.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.000001);
  delete m; delete ms;
  delete d; delete ds;
}
