/**
 * Performs the dot-product test
 * for the second derivative
 * operator
 * @author: Joseph Jennings
 * @version: 2016.12.29
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/scndder.h"
#include "axis.h"
#include "hypercube_float.h"

class scnddertest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(scnddertest,dptest){
  int nx = 100; int nz = 100;
  int nts = 250; float dts  = 0.004;
  /* Build axes */
  axis ax  = axis(nx,  0.0, 1.0);
  axis az  = axis(nz,  0.0, 1.0);
  axis ats = axis(nts, 0.0, dts);
  std::vector<axis> uaxes;
  uaxes.push_back(az); uaxes.push_back(ax);
  uaxes.push_back(ats);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(uaxes, true);
  hypercube_float *ms = new hypercube_float(uaxes, true);
  hypercube_float *d  = new hypercube_float(uaxes, true);
  hypercube_float *ds = new hypercube_float(uaxes, true);
  m->random(); d->random();
  // I think this is cheating...
  m->scale(0.01); d->scale(0.01);
  scndder sdr = scndder(ats);
  /* Add == false */
  sdr.forward(false, m, ds);
  float dotd = d->dot(ds);
  sdr.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.00001);
  /* Add == true */
  sdr.forward(true,m,ds);
  float adotd = d->dot(ds);
  sdr.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.00001);
  delete m; delete ms;
  delete d; delete ds;
}

