/**
 * Tests the source scale operator
 * @author: Joseph Jennings
 * @version: 2016.11.24
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/sourcescale.h"
#include "axis.h"
#include "hypercube_float.h"

class sourcescaletest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(sourcescaletest,dptest){
  int nx = 100; int nz = 100;
  int nt = 250; float dtu = 0.004;
  /* Build axes */
  axis ax = axis(nx,  0.0, 1.0);
  axis az = axis(nz,  0.0, 1.0);
  axis at = axis(nt, 0.0, 1.0);
  std::vector<axis> paxes; std::vector<axis> vaxes;
  paxes.push_back(az); paxes.push_back(ax);
  paxes.push_back(at);
  vaxes.push_back(az); vaxes.push_back(ax);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(paxes, true);
  hypercube_float *ms = new hypercube_float(paxes, true);
  hypercube_float *d  = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  hypercube_float *v  = new hypercube_float(vaxes, true);
  m->random(); d->random(); v->random();
  sourcescale ss = sourcescale(dtu,v);
  /* Add == false */
  ss.forward(false, m, ds);
  float dotd = d->dot(ds);
  ss.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  /* Add == true */
  ss.forward(true,m,ds);
  float adotd = d->dot(ds);
  ss.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.000001);
  delete v;
  delete m; delete ms;
  delete d; delete ds;
}
