/**
 * Tests the cosine taper class
 * for absorbing boundary wave
 * propagation
 * @author: Joseph Jennings
 * @version: 2017.01.04
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/tapercos.h"
#include "../main/zeropadimg.h"
#include "../main/simpleplot.h"
#include "axis.h"
#include "hypercube_float.h"

class tapercostest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(tapercostest,tapercomp){
  int nx = 200; int nz = 200;
  int p = 4; int shift = p/2;
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az); axes.push_back(ax);
  paxes.push_back(azp); paxes.push_back(axp);
  /* Build hypercubes */
  hypercube_float *ones  = new hypercube_float(axes ,true);
  hypercube_float *pones = new hypercube_float(paxes,true);
  hypercube_float *zero  = new hypercube_float(paxes,true);
  ones->set_val(1.0);
  zeropadimg zp = zeropadimg(p/2,p/2,0.0);
  zp.forward(false,ones,pones);
  tapercos tap = tapercos(paxes, 50, 50, p, 0.97);
  tap.apply(pones,zero);
  hypercube_float *btap = tap.get_taper();
  pones->scale_add(1.0,btap,-1.0);
  double psum = pones->sum();
  ASSERT_EQ(psum,0.0);
  delete ones; delete pones; delete zero;
}

TEST_F(tapercostest,nopadcomp) {
  int nx = 200; int nz = 200;
  int p = 4; int shift = p/2;
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az); axes.push_back(ax);
  paxes.push_back(azp); paxes.push_back(axp);
  /* Build hypercubes */
  hypercube_float *r1  = new hypercube_float(axes ,true);
  hypercube_float *zr  = new hypercube_float(paxes,true);
  hypercube_float *pr1 = new hypercube_float(paxes,true);
  hypercube_float *pzr = new hypercube_float(paxes,true);
  hypercube_float *tr1 = new hypercube_float(axes, true);
  r1->random(); pr1->random();
  zeropadimg zp = zeropadimg(p/2,p/2,0.0);
  zp.forward(false,r1,pr1);
  tapercos tap  = tapercos(axes ,50,50,0,0.97);
  tapercos ptap = tapercos(paxes,50,50,p,0.97);
  ptap.apply(pr1,pzr); tap.apply(r1,zr);
  zp.adjoint(false,tr1,pr1);
  simpleplot::plot2d(tr1); simpleplot::plot2d(r1);
  double psum = tr1->sum();
  double  sum = r1->sum();
  ASSERT_EQ(sum,psum);
}
