/**
 * Tests functions within the
 * wavelet class
 * @author: Joseph Jennings
 * @version: 2016.12.11
 */

#include <limits.h>
#include <algorithm>
#include <complex>
#include <fftw3.h>
#include "gtest/gtest.h"
#include "../main/wavelet.h"
#include "axis.h"
#include "hypercube_float.h"

class wavelettest: public ::testing::Test {
  public:
    hypercube_float *ampspec(hypercube_float *sig);
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(wavelettest,bpanatest){
  int nt = 128; float dt = static_cast<float>(1.0/nt);
  float ot = 0.0;
  std::vector<axis> at; at.push_back(axis(nt,ot,dt));
  hypercube_float *sig  = new hypercube_float(at, true);
  hypercube_float *fsig = new hypercube_float(at,true);

  wavelet wav = wavelet(at[0]);
  std::vector<float> fs = {10.0,40.0,50.0};
  std::vector<float> as = {1.0,1.0,1.0};
  std::vector<float> ps = {0.0,90.0,0.0};
  wav.sumsin(fs,as,ps,sig);
  fsig->set(sig->vals);

  std::vector<float> ffs = {1.0,5.0,15.0,19.0};
  wav.bandpass(ffs,fsig);

  hypercube_float *spec = ampspec(fsig);
  int nw = spec->get_axis(1).n;
  int lrgfreq = nw - 1;
  /* Find location of maximum */
  for(int iw = nw-2; iw >= 0; --iw) {
    if(spec->vals[iw] > spec->vals[lrgfreq]) lrgfreq = iw;
  }
  ASSERT_EQ(lrgfreq,10);

  delete sig; delete fsig;
}

TEST_F(wavelettest,randtest) {
  int nt = 256; float dt = static_cast<float>(1.0/nt);
  float ot = 0.0;
  std::vector<axis> at; at.push_back(axis(nt,ot,dt));
  hypercube_float *sig  = new hypercube_float(at, true);
  hypercube_float *fsig = new hypercube_float(at,true);

  wavelet wav = wavelet(at[0]);
  std::vector<float> fs = {1.0,5.0,45.0,59.0};
  wav.bprandom(fs,sig);
  float sum = 0.0;
  hypercube_float *spec = ampspec(sig);
  for(int iw = 60; iw < spec->get_axis(1).n; ++iw) {sum += spec->vals[iw];}

  ASSERT_LE(sum,0.00001);
}

hypercube_float *wavelettest::ampspec(hypercube_float *sig) {
  /* FFT of filtered signal*/
  int nt = sig->get_axis(1).n; float dt = sig->get_axis(1).d;
  std::complex<double> *in  = new std::complex<double>[nt];
  std::complex<double> *out = new std::complex<double>[nt];
  for(int it = 0; it < nt; ++it) in[it].real(sig->vals[it]);
  fftw_plan p = fftw_plan_dft_1d(nt,reinterpret_cast<fftw_complex*>(in),
      reinterpret_cast<fftw_complex*>(out),FFTW_FORWARD,FFTW_ESTIMATE);

  fftw_execute(p);

  /* Amplitude spectrum */
  int nw = nt/2; float dw = static_cast<float>(1.0/(nt*dt));
  std::vector<axis> aw; aw.push_back(axis(nw,0.0,dw));
  hypercube_float *spec = new hypercube_float(aw,true);
  for(int iw = 0; iw < nw; ++iw) {
    spec->vals[iw] = sqrt(out[iw].real()*out[iw].real() + out[iw].imag()*out[iw].imag());
  }
  return spec;
}
