/**
 * Performs the dot-product test
 * for the wavefield interpolation
 * operator
 * @author: Joseph Jennings
 * @version: 2016.11.24
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/wfldlint.h"
#include "axis.h"
#include "hypercube_float.h"

class wfldlinttest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(wfldlinttest,dptest){
  int nx = 100; int nz = 100;
  int nts = 250; float dts  = 0.004;
  int ntu = 1000; float dtu = 0.001;
  /* Build axes */
  axis ax  = axis(nx,  0.0, 1.0);
  axis az  = axis(nz,  0.0, 1.0);
  axis ats = axis(nts, 0.0, dts);
  axis atu = axis(ntu, 0.0, dtu);
  std::vector<axis> saxes; std::vector<axis> uaxes;
  saxes.push_back(az); saxes.push_back(ax);
  saxes.push_back(ats);
  uaxes.push_back(az); uaxes.push_back(ax);
  uaxes.push_back(atu);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(saxes, true);
  hypercube_float *ms = new hypercube_float(saxes, true);
  hypercube_float *d  = new hypercube_float(uaxes, true);
  hypercube_float *ds = new hypercube_float(uaxes, true);
  m->random(); d->random();
  wfldlint wi = wfldlint();
  /* Add == false */
  wi.forward(false, m, ds);
  float dotd = d->dot(ds);
  wi.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.0001);
  /* Add == true */
  wi.forward(true,m,ds);
  float adotd = d->dot(ds);
  wi.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.0001);
  delete m; delete ms;
  delete d; delete ds;
}
