/**
 * Tests the zero pad operator
 * @author: Joseph Jennings
 * @version: 2016.11.24
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/zeropad.h"
#include "axis.h"
#include "hypercube_float.h"

class zeropadtest : public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(zeropadtest,dptest){
  int nx = 100; int nz = 100;
  int nts = 250;
  int xs = 50; int zs = 0;
  /* Build axes */
  axis ax = axis(nx,  0.0, 1.0);
  axis az = axis(nz,  0.0, 1.0);
  axis at = axis(nts, 0.0, 1.0);
  std::vector<axis> paxes; std::vector<axis> saxis;
  saxis.push_back(at); paxes.push_back(ax);
  paxes.push_back(az); paxes.push_back(at);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(saxis, true);
  hypercube_float *ms = new hypercube_float(saxis, true);
  hypercube_float *d  = new hypercube_float(paxes, true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  m->random(); d->random();
  zeropad zp = zeropad(xs,zs);
  /* Add == false */
  zp.forward(false, m, ds);
  float dotd = d->dot(ds);
  zp.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  /* Add == true */
  zp.forward(true,m,ds);
  float adotd = d->dot(ds);
  zp.adjoint(true,ms,d);
  float adotm = m->dot(ms);
  EXPECT_NEAR(adotd, adotm, 0.000001);
  delete m; delete ms;
  delete d; delete ds;
}
