/**
 * Tests the Zero Pad wavefield operator
 * @author: Joseph Jennings
 * @version: 2016.12.13
 */

#include <limits.h>
#include "gtest/gtest.h"
#include "../main/zeropadwfld.h"
#include "../main/simpleplot.h"
#include "axis.h"
#include "hypercube_float.h"

class zeropadwfldtest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(zeropadwfldtest,dptest){
  int nx = 100; int nz = 100; int nt = 20;
  int p = 50; int shift = p/2;
  /* Build axes */
  axis ax  = axis(nx  , 0.0, 1.0);
  axis az  = axis(nz  , 0.0, 1.0);
  axis axp = axis(nx+p, 0.0, 1.0);
  axis azp = axis(nz+p, 0.0, 1.0);
  axis at  = axis(nt,   0.0, 1.0);
  std::vector<axis> axes, paxes;
  axes.push_back(az);   axes.push_back(ax); axes.push_back(at);
  paxes.push_back(azp); paxes.push_back(axp); paxes.push_back(at);
  /* Build hypercubes */
  hypercube_float *m  = new hypercube_float(axes , true);
  hypercube_float *ms = new hypercube_float(axes, true);
  hypercube_float *d  = new hypercube_float(paxes , true);
  hypercube_float *ds = new hypercube_float(paxes, true);
  m->random(); d->random();
  zeropadwfld zpw = zeropadwfld(shift,shift,0.0);
  /* Add == false */
  zpw.forward(false, m, ds);
  float dotd = d->dot(ds);
  zpw.adjoint(false, ms, d);
  float dotm = m->dot(ms);
  EXPECT_NEAR(dotd, dotm, 0.000001);
  delete m; delete ms;
  delete d; delete ds;
}
